"""dist-git support for Revumatic"""

import re

from modules import packages


class DistGit(packages.Package):
    """Dist-git handling class."""
    paths = (
        'redhat/centos-stream/rpms/',
        'redhat/rhel/rpms/',
        'redhat/centos-stream/containers/',
        'redhat/rhel/containers/',
        'redhat/centos-stream/tests/',
        'redhat/rhel/tests/'
    )

    def __init__(self):
        """Set up defaults."""
        self._jira_ticket_prefix = "https://issues.redhat.com/browse/"

        # All of the below are valid:
        #
        # Resolves: RHEL-1
        # Resolves:RHEL-1
        # Resolve: RHEL-11, RHEL-22
        # - Resolves: RHEL-33
        #
        # Also, Resolve*s*, Revert*s* vs Relate*d* is very intuitive...
        self._tickets_re = re.compile(
            r"^\s*-?\s*(?:Resolves?|Related?|Reverts?):\s*(.*)",
            re.I | re.M
        )

    def get_bug_urls(self, mr, commits):
        """Get a list of all referenced Jira tickets.

        We also have to handle multiple tickets on a single line, such as

            Resolves: RHEL-11, RHEL-111
        """
        tickets = []

        raw_matches = self._tickets_re.findall(mr['description'])
        for (commit, _) in commits.commit_getter.commits:
            raw_matches.extend(self._tickets_re.findall(commit.message))

        for match in raw_matches:
            tickets.extend(
                self._jira_ticket_prefix + ticket.strip(' ,')
                for ticket in match.split(',')
            )

        return tickets


packages.add(DistGit)
