import argparse
import tuimatic
from collections.abc import Sequence
from typing import Optional, Union, TYPE_CHECKING
if TYPE_CHECKING:
    from modules import app

class ScreenError(Exception):
    pass


class BaseScreen:
    command: Optional[str] = None
    help: Optional[str] = None
    help_epilog: Optional[str] = None
    help_raw: bool = False
    ui_arg: Optional[dict['str', Union['str', type, tuple['str', ...]]]] = None

    @classmethod
    def add_arguments(cls, parser: argparse.ArgumentParser) -> None:
        return

    @classmethod
    def register_settings(cls) -> None:
        return

    def __init__(self, app: 'app.App', args: argparse.Namespace) -> None:
        class Widgets:
            pass

        self.widgets = Widgets()
        self.app = app
        self.action_tracker = self.app.new_action_tracker(self.action)

    def start(self) -> None:
        widget = self.init_ui()
        self.app.set_workplace(widget)
        self.load_data()

    def init_ui(self) -> Optional[tuimatic.Widget]:
        return None

    def load_data(self) -> None:
        pass

    def get_long_help(self) -> Sequence[str]:
        return ()

    def restored(self) -> None:
        """Called when a screen that we started exited and we're active again."""
        return

    def action(self, what: str, widget: tuimatic.Widget, size: tuple[int, ...]) -> None:
        pass

    def keypress(self, key: str) -> bool:
        return False
