from typing import Optional

class ThemeBase:
    palette: Optional[tuple[tuple[str, str, str], ...]] = None


class BlackTheme(ThemeBase):
    palette = (
        ('std', 'light gray', 'black'),
        ('dimmed', 'dark gray', 'black'),
        ('button-shortcut', 'light cyan', 'black'),
        ('select', 'black', 'light gray'),
        ('title', 'white', 'black'),
        ('progress', 'white', 'dark blue'),
        ('divider', 'white', 'black'),
        ('header', 'black', 'light gray'),
        ('header-branch', 'black', 'light blue'),
        ('header-commits', 'black', 'light cyan'),
        ('header-version', 'black', 'dark cyan'),
        ('header-problem', 'dark red', 'light gray'),
        ('header-info', 'brown', 'light gray'),
        ('subheader', 'white', 'dark gray'),
        ('subheader-commit', 'light cyan', 'dark gray'),
        ('footer', 'black', 'light gray'),
        ('help-key', 'white', 'dark gray'),
        ('help-desc', 'black', 'light gray'),
        ('desc', 'light gray', 'black'),
        ('desc-header', 'white', 'black'),
        ('desc-header-approved', 'light green', 'black'),
        ('desc-header-blocked', 'light red', 'black'),
        ('desc-date', 'white', 'black'),
        ('desc-person', 'light cyan', 'black'),
        ('desc-username', 'light gray', 'black'),
        ('desc-me', 'yellow', 'black'),
        ('desc-version', 'light cyan', 'black'),
        ('desc-sys', 'dark gray', 'black'),
        ('desc-ok', 'light green', 'black'),
        ('desc-err', 'light red', 'black'),
        ('desc-warn', 'yellow', 'black'),
        ('desc-label', 'white', 'light blue'),
        ('desc-label-ok', 'white', 'dark green'),
        ('desc-label-warn', 'white', 'brown'),
        ('desc-label-err', 'white', 'dark red'),
        ('diff', 'light gray', 'black'),
        ('diff-file', 'white', 'black'),
        ('diff-hunk', 'yellow', 'black'),
        ('diff-func', 'dark cyan', 'black'),
        ('diff-add', 'dark green', 'black'),
        ('diff-del', 'dark red', 'black'),
        ('diff-miss', 'white', 'dark red'),
        ('diff-add-miss', 'light green', 'dark red'),
        ('diff-del-miss', 'light red', 'dark red'),
        ('diff-subject', 'black', 'dark cyan'),
        ('diff-sha', 'light cyan', 'black'),
        ('diff-commit-ref', 'dark gray', 'black'),
        ('diff-quote-char', 'dark cyan', 'black'),
        ('list', 'light gray', 'black'),
        ('list-error', 'light red', 'black'),
        ('list-warning', 'brown', 'black'),
        ('list-info', 'yellow', 'black'),
        ('list-sha', 'light cyan', 'black'),
        ('list-id', 'dark cyan', 'black'),
        ('list-date', 'white', 'black'),
        ('list-time', 'dark gray', 'black'),
        ('list-person', 'light blue', 'black'),
        ('list-comment', 'dark gray', 'black'),
        ('list-label', 'light gray', 'black'),
        ('list-label-ok', 'dark green', 'black'),
        ('list-label-warn', 'brown', 'black'),
        ('list-label-err', 'light red', 'black'),
        ('list-item-important', 'light cyan', 'black'),
        ('a:std', 'white', 'dark blue'),
        ('a:button-shortcut', 'light cyan', 'dark blue'),
        ('a:list', 'white', 'dark blue'),
        ('a:list-error', 'light red', 'dark blue'),
        ('a:list-warning', 'brown', 'dark blue'),
        ('a:list-info', 'yellow', 'dark blue'),
        ('a:list-sha', 'light cyan', 'dark blue'),
        ('a:list-id', 'dark cyan', 'dark blue'),
        ('a:list-date', 'white', 'dark blue'),
        ('a:list-time', 'light gray', 'dark blue'),
        ('a:list-person', 'light blue', 'dark blue'),
        ('a:list-comment', 'light gray', 'dark blue'),
        ('a:list-label', 'white', 'dark blue'),
        ('a:list-label-ok', 'dark green', 'dark blue'),
        ('a:list-label-warn', 'brown', 'dark blue'),
        ('a:list-label-err', 'light red', 'dark blue'),
        ('a:list-item-important', 'light cyan', 'dark blue'),
        ('p:std', 'white', 'black'),
        ('p:button-shortcut', 'light cyan', 'black'),
        ('p:list', 'white', 'black'),
        ('p:list-error', 'light red', 'black'),
        ('p:list-warning', 'brown', 'black'),
        ('p:list-info', 'yellow', 'black'),
        ('p:list-sha', 'light cyan', 'black'),
        ('p:list-id', 'dark cyan', 'black'),
        ('p:list-date', 'white', 'black'),
        ('p:list-time', 'dark gray', 'black'),
        ('p:list-person', 'light blue', 'black'),
        ('p:list-comment', 'dark gray', 'black'),
        ('p:list-label', 'light gray', 'black'),
        ('p:list-label-ok', 'white', 'black'),
        ('p:list-label-warn', 'brown', 'black'),
        ('p:list-label-err', 'light red', 'black'),
        ('p:list-item-important', 'light cyan', 'black'),
        ('h:std', 'white', 'brown'),
        ('h:diff', 'white', 'brown'),
        ('h:diff-func', 'light cyan', 'brown'),
        ('h:diff-add', 'dark green', 'brown'),
        ('h:diff-del', 'dark red', 'brown'),
        ('h:diff-sha', 'light cyan', 'brown'),
        ('h:diff-commit-ref', 'light gray', 'brown'),
        ('h:diff-quote-char', 'light cyan', 'brown'),
    )


class SolarizedLightTheme(ThemeBase):
    palette = (
        ('std', 'default', 'default'),
        ('dimmed', 'default', 'default'),
        ('button-shortcut', 'brown', 'default'),
        ('select', 'default', 'default'),

        ('title', 'dark gray', 'default'),
        ('progress', 'black', 'light cyan'),
        ('divider', 'dark gray', 'default'),
        ('header', 'white', 'dark blue'),
        ('header-branch', 'yellow', 'light blue'),
        ('header-commits', 'yellow', 'light blue'),
        ('header-version', 'yellow', 'light blue'),
        ('header-problem', 'dark red', 'light gray'),
        ('header-info', 'brown', 'light gray'),
        ('subheader', 'black', 'default'),
        ('subheader-commit', 'brown', 'default'),
        ('footer', 'black', 'light gray'),
        ('help-key', 'white', 'dark gray'),
        ('help-desc', 'black', 'light gray'),

        ('desc', 'black', 'default'),
        ('desc-header', 'black', 'default'),
        ('desc-header-approved', 'black', 'default'),
        ('desc-header-blocked', 'black', 'default'),
        ('desc-date', 'dark gray', 'default'),
        ('desc-person', 'dark blue', 'default'),
        ('desc-username', 'black', 'default'),
        ('desc-me', 'dark green', 'default'),
        ('desc-version', 'dark cyan, bold', 'default'),
        ('desc-sys', 'dark gray', 'default'),
        ('desc-ok', 'dark green', 'default'),
        ('desc-err', 'dark red', 'default'),
        ('desc-warn', 'brown', 'default'),
        ('desc-label', 'white', 'light cyan'),
        ('desc-label-ok', 'white', 'light green'),
        ('desc-label-warn', 'dark red, bold', 'yellow'),
        ('desc-label-err', 'yellow', 'light red'),

        ('diff', 'black', 'default'),
        ('diff-file', 'light green', 'default'),
        ('diff-hunk', 'brown', 'default'),
        ('diff-func', 'dark magenta', 'default'),
        ('diff-add', 'dark green', 'default'),
        ('diff-del', 'dark red', 'default'),
        ('diff-miss', 'yellow, bold', 'dark red'),
        ('diff-add-miss', 'yellow, bold', 'dark red'),
        ('diff-del-miss', 'yellow, bold', 'dark red'),
        ('diff-subject', 'black', 'light cyan'),
        ('diff-sha', 'brown', 'default'),
        ('diff-commit-ref', 'dark gray', 'default'),
        ('diff-quote-char', 'dark magenta', 'default'),

        ('list', 'default', 'default'),
        ('list-error', 'dark red', 'default'),
        ('list-warning', 'brown', 'default'),
        ('list-info', 'dark green', 'default'),
        ('list-sha', 'brown', 'default'),
        ('list-id', 'dark magenta', 'default'),
        ('list-date', 'dark gray', 'default'),
        ('list-time', 'dark gray', 'default'),
        ('list-person', 'dark blue', 'default'),
        ('list-comment', 'dark gray', 'default'),
        ('list-label', 'light gray', 'default'),
        ('list-label-ok', 'default', 'default'),
        ('list-label-warn', 'brown', 'default'),
        ('list-label-err', 'dark red', 'default'),
        ('list-item-important', 'brown', 'default'),
        ('a:std', 'black', 'light gray'),
        ('a:button-shortcut', 'brown', 'light gray'),
        ('a:list', 'black', 'light gray'),
        ('a:list-error', 'dark red', 'light gray'),
        ('a:list-warning', 'brown', 'light gray'),
        ('a:list-info', 'dark green', 'light gray'),
        ('a:list-sha', 'brown', 'light gray'),
        ('a:list-id', 'dark magenta', 'light gray'),
        ('a:list-date', 'dark gray', 'light gray'),
        ('a:list-time', 'dark gray', 'light gray'),
        ('a:list-person', 'dark blue', 'light gray'),
        ('a:list-comment', 'light gray', 'light gray'),
        ('a:list-label', 'default', 'light gray'),
        ('a:list-label-ok', 'default', 'light gray'),
        ('a:list-label-warn', 'brown', 'light gray'),
        ('a:list-label-err', 'dark red', 'light gray'),
        ('a:list-item-important', 'brown', 'light gray'),
        ('p:std', 'black', 'light gray'),
        ('p:button-shortcut', 'brown', 'light gray'),
        ('p:list', 'black', 'light gray'),
        ('p:list-error', 'light red', 'light gray'),
        ('p:list-warning', 'brown', 'light gray'),
        ('p:list-info', 'dark green', 'light gray'),
        ('p:list-sha', 'brown', 'light gray'),
        ('p:list-id', 'dark magenta', 'light gray'),
        ('p:list-date', 'dark gray', 'light gray'),
        ('p:list-time', 'dark gray', 'light gray'),
        ('p:list-person', 'dark blue', 'light gray'),
        ('p:list-comment', 'dark gray', 'light gray'),
        ('p:list-label', 'light gray', 'light gray'),
        ('p:list-label-ok', 'black', 'light gray'),
        ('p:list-label-warn', 'brown', 'light gray'),
        ('p:list-label-err', 'light red', 'light gray'),
        ('p:list-item-important', 'brown', 'light gray'),

        ('h:std', 'yellow', 'light red'),
        ('h:diff', 'black', 'light red'),
        ('h:diff-func', 'brown', 'light red'),
        ('h:diff-add', 'dark green', 'light red'),
        ('h:diff-del', 'dark red', 'light red'),
        ('h:diff-sha', 'brown', 'light red'),
        ('h:diff-commit-ref', 'dark gray', 'light red'),
        ('h:diff-quote-char', 'dark magenta', 'light red'),
    )
